import os,sys,threading,pty,logging

import gtk, gobject, pango
import vte
import gconf

import hotwire_ui.widgets as hotwidgets
from hotwire.sysdep.term_impl.base_term import BaseTerminal, TerminalWidget
from hotwire.async import MiniThreadPool

_logger = logging.getLogger("hotwire.sysdep.Terminal.Vte")

class VteTerminal(BaseTerminal):
    def get_terminal_widget(self, stream, title):
        return VteTerminalWidget(stream=stream, title=title)

    def get_terminal_widget_cmd(self, cwd, cmd, title):
        return VteTerminalWidget(cwd=cwd, cmd=cmd, title=title)

class VteTerminalWidget(TerminalWidget):
    def __init__(self, cwd=None, cmd=None, **kwargs):
        super(VteTerminalWidget, self).__init__(**kwargs)
        self.__termbox = gtk.HBox()
        self.__term = vte.Terminal()
        self.__scroll = gtk.VScrollbar(self.__term.get_adjustment())
        self.__termbox.pack_start(hotwidgets.Border(self.__term))
        self.__termbox.pack_start(self.__scroll, False)

        self._pack_terminal(self.__termbox)

	self.__term.set_emulation('xterm')
	self.__term.set_allow_bold(True)

        gconf_client = gconf.client_get_default() 
        mono_font = gconf_client.get_string('/desktop/gnome/interface/monospace_font_name')
        _logger.debug("Using font '%s'", mono_font)
        font_desc = pango.FontDescription(mono_font)
        self.__term.set_font(font_desc)
        fg = self.__term.style.text[gtk.STATE_NORMAL]
        bg = self.__term.style.base[gtk.STATE_NORMAL]

        self.__term.set_default_colors()
        self.__term.set_color_background(bg)
        self.__term.set_color_foreground(fg)

        if self._stream:
            master, slave = pty.openpty()
            self.__master = master
            self.__slave = slave
            self.__term.set_pty(self.__master)
            _logger.debug("Created pty master: %d slave: %d", master, slave)
            MiniThreadPool.getInstance().run(self.__fd_to_stream, args=(self.__slave, self._stream))
            MiniThreadPool.getInstance().run(self.__stream_to_fd, args=(self._stream, self.__slave))
        else:
            self._stream = None
            _logger.debug("Forking cmd: %s", cmd)
            self.__term.connect("child-exited", self._on_child_exited)
            if cmd:
                pid = self.__term.fork_command('/bin/sh', ['/bin/sh', '-c', cmd], directory=cwd)
            else:
                pid = self.__term.fork_command(directory=cwd)
            self._set_pid(pid)

    def __fd_to_stream(self, fd, stream):
        while True:
            buf = os.read(fd,4096)
            if buf == '':
                break
            stream.send(buf)

    def __stream_to_fd(self, stream, fd):
        while True:
            buf = stream.recv(4096)
            if buf == '':
                break
            os.write(fd, buf)
        gobject.idle_add(lambda: self.close())

    def close(self):
        if self._stream:
            _logger.debug("Closing stream")
            self._stream.close()
            os.close(self.__master)
            os.close(self.__slave)
            self._stream = None

    def do_dispose(self):
        self.__term = None

    def copy(self):
        _logger.debug("got copy")
        self.__term.copy_clipboard()

    def paste(self):
        _logger.debug("got paste")
        self.__term.paste_clipboard()

def getInstance():
    return VteTerminal()
